﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Collections.ObjectModel;
using LinqToVisualTree;
using System.Windows.Controls.Primitives;
using Microsoft.Xna.Framework;
using System.IO;
using Microsoft.Xna.Framework.Audio;
using SystemColor = System.Windows.Media.Color;

namespace ClearStyle
{
  public partial class MainPage : PhoneApplicationPage
  {
    private RangeObservableCollection<ToDoItem> _todoItems = new RangeObservableCollection<ToDoItem>();

    private bool _flickOccured;

    private Canvas _tickAndCrossContainer;

    private static double FLICK_VELOCITY = 2000.0;

    SoundEffect _completeSound;
    SoundEffect _deleteSound;

    // Constructor
    public MainPage()
    {
      InitializeComponent();

      _completeSound = SoundEffect.FromStream(TitleContainer.OpenStream("Sounds/Windows XP Exclamation.wav"));
      _deleteSound = SoundEffect.FromStream(TitleContainer.OpenStream("Sounds/Windows XP Notify.wav"));
      FrameworkDispatcher.Update();

      _todoItems.Add(new ToDoItem("Feed the cat"));
      _todoItems.Add(new ToDoItem("Buy eggs"));
      _todoItems.Add(new ToDoItem("Pack bags for WWDC conference"));
      _todoItems.Add(new ToDoItem("Rule the web"));
      _todoItems.Add(new ToDoItem("Order business cards"));
      _todoItems.Add(new ToDoItem("Fix laptop"));
      _todoItems.Add(new ToDoItem("Get some dollars for trip"));
      _todoItems.Add(new ToDoItem("Shirts"));
      _todoItems.Add(new ToDoItem("Shopping"));
      _todoItems.Add(new ToDoItem("Contact PR company"));
      _todoItems.Add(new ToDoItem("Extension plans"));
      _todoItems.Add(new ToDoItem("Choose colour scheme"));
      _todoItems.Add(new ToDoItem("Create new website"));
      _todoItems.Add(new ToDoItem("Write-up blog post"));
      _todoItems.Add(new ToDoItem("Choose life"));
      _todoItems.Add(new ToDoItem("Simplify my life"));

      UpdateToDoColors();
      this.DataContext = _todoItems;
    }

    private void UpdateToDoColors()
    {
      double itemCount = _todoItems.Count;
      double index = 0;
      foreach (var todoItem in _todoItems)
      {
        double val = (index / itemCount) * 155.0;
        index++;

        if (!todoItem.Completed)
        {
          todoItem.Color = SystemColor.FromArgb(255, 255, (byte)val, 0);
        }
      };         
    }

    private double TickAndCrossOpacity(double offset)
    {
      offset = Math.Abs(offset);
      if (offset < 50)
        return 0;

      offset -= 50;
      double opacity = offset / 100;

      opacity = Math.Max(Math.Min(opacity, 1), 0);
      return opacity;
    }

    private void GestureListener_DragStarted(object sender, DragStartedGestureEventArgs e)
    {
      // initialize the drag
      FrameworkElement fe = sender as FrameworkElement;
      fe.SetHorizontalOffset(0);
      _flickOccured = false;

      // find the container for the tick and cross graphics
      _tickAndCrossContainer = fe.Descendants()
                                 .OfType<Canvas>()
                                 .Single(i => i.Name == "tickAndCross");
    }

    private void GestureListener_DragDelta(object sender, DragDeltaGestureEventArgs e)
    {
      // handle the drag to offset the element
      FrameworkElement fe = sender as FrameworkElement;
      double offset = fe.GetHorizontalOffset().Value + e.HorizontalChange;
      fe.SetHorizontalOffset(offset);


      _tickAndCrossContainer.Opacity = TickAndCrossOpacity(offset);
    }

    private void GestureListener_DragCompleted(object sender, DragCompletedGestureEventArgs e)
    {
      if (_flickOccured)
        return;

      FrameworkElement fe = sender as FrameworkElement;
      if (Math.Abs(e.HorizontalChange) > fe.ActualWidth / 2)
      {
        if (e.HorizontalChange < 0.0)
        {
          ToDoItemDeletedAction(fe);  
        }
        else
        {
          ToDoItemCompletedAction(fe);
        }
      }
      else
      {
        ToDoItemBounceBack(fe);
      }
    }

    private void ToDoItemBounceBack(FrameworkElement fe)
    {
      var trans = fe.GetHorizontalOffset().Transform;

      trans.Animate(trans.X, 0, TranslateTransform.XProperty, 300, 0, new BounceEase()
      {
        Bounciness = 5,
        Bounces = 2
      });
    }

    private void ToDoItemDeletedAction(FrameworkElement deletedElement)
    {
      _deleteSound.Play();

      var trans = deletedElement.GetHorizontalOffset().Transform;
      trans.Animate(trans.X, -(deletedElement.ActualWidth + 50),
                    TranslateTransform.XProperty, 300, 0, new SineEase()
      {
        EasingMode = EasingMode.EaseOut
      },
      () =>
      {
        // find the model object that was deleted
        ToDoItem deletedItem = deletedElement.DataContext as ToDoItem;
        
        // determine how much we have to 'shuffle' up by
        double elementOffset = -deletedElement.ActualHeight;

        // find the items in view, and the location of the deleted item in this list
        var itemsInView = todoList.GetItemsInView().ToList();
        var lastItem = itemsInView.Last();
        int startTime = 0;
        int deletedItemIndex = itemsInView.Select(i => i.DataContext)
                                          .ToList().IndexOf(deletedItem);

        // iterate over each item
        foreach (FrameworkElement element in itemsInView.Skip(deletedItemIndex))
        {
          // for the last item, create an action that deletes the model object
          // and re-renders the list
          Action action = null;
          if (element == lastItem)
          {
            action = () =>
            {
              // clone the list
              var items = _todoItems.ToList();
              items.Remove(deletedItem);

              // re-populate our ObservableCollection
              _todoItems.Clear();
              _todoItems.AddRange(items);
              UpdateToDoColors();
            };
          }

          // shuffle this item up
          TranslateTransform elementTrans = new TranslateTransform();
          element.RenderTransform = elementTrans;
          elementTrans.Animate(0, elementOffset, TranslateTransform.YProperty, 200, startTime, null, action);
          startTime += 10;
        }
      });
    }

    private void ToDoItemCompletedAction(FrameworkElement fe)
    {
      // set the mode object to complete
      ToDoItem completedItem = fe.DataContext as ToDoItem;
      completedItem.Completed = true;
      completedItem.Color = Colors.Green;

      // bounce back into place
      ToDoItemBounceBack(fe);

      _completeSound.Play();
    }

    private void GestureListener_Flick(object sender, FlickGestureEventArgs e)
    {
      FrameworkElement fe = sender as FrameworkElement;
      if (e.HorizontalVelocity < -FLICK_VELOCITY)
      {
        _flickOccured = true;
        ToDoItemDeletedAction(fe);
      }
      else if (e.HorizontalVelocity > FLICK_VELOCITY)
      {
        _flickOccured = true;
        ToDoItemCompletedAction(fe);        
      }
    }
  }
}