﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace ClearStyle
{
  public static class Util
  {
    public static void Animate(this DependencyObject target, double from, double to,
                              object propertyPath, int duration, int startTime,
                              IEasingFunction easing = null, Action completed = null)
    {
      if (easing == null)
      {
        easing = new SineEase();
      }

      var db = new DoubleAnimation();
      db.To = to;
      db.From = from;
      db.EasingFunction = easing;
      db.Duration = TimeSpan.FromMilliseconds(duration);
      Storyboard.SetTarget(db, target);
      Storyboard.SetTargetProperty(db, new PropertyPath(propertyPath));

      var sb = new Storyboard();
      sb.BeginTime = TimeSpan.FromMilliseconds(startTime);

      if (completed != null)
      {
        sb.Completed += (s, e) => completed();
      }

      sb.Children.Add(db);
      sb.Begin();
    }

    public static void SetHorizontalOffset(this FrameworkElement fe, double offset)
    {
      var trans = new TranslateTransform()
      {
        X = offset
      };
      fe.RenderTransform = trans;

      fe.Tag = new Offset()
        {
          Value = offset,
          Transform  = trans
        };
    }

    public static Offset GetHorizontalOffset(this FrameworkElement fe)
    {
      return fe.Tag == null ? new Offset() : (Offset)fe.Tag;
    }

    public struct Offset
    {
      public double Value { get; set; }
      public TranslateTransform Transform { get; set; }
    }
  }
}
